/* Copyright (C) 2007 - 2008 MySQL AB, 2008 Sun Microsystems, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License.

   There are special exceptions to the terms and conditions of the GPL
   as it is applied to this software. View the full text of the
   exception in file EXCEPTIONS-CONNECTOR-C++ in the directory of this
   software distribution.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "StatementTest.h"

#include "../driver/mysql_resultset.h"

namespace testsuite
{
namespace compliance
{

    /**
     * Constructor for StatementTest.
     * @param name
     */
  StatementTest::StatementTest(String name)
             : super(name)
  {
  }

    /*
     * @testName:         testClearWarnings
     * @assertion:        The Statement object provides methods for executing SQL
     *            statements and retrieving results.(See section 40.1 of
     *            JDBC 2.0 API Reference & Tutorial second edition).
     *
     *            The driver must provide full support for Statement methods.
     *            The driver must also support all the methods for executing
     *            SQL Statements in a single batch (Batch Updates). (See
     *            section 6.2.2.3 of Java2 Platform Enterprise Edition(J2EE)
     *            Specification v1.2)
     *
     *            The ClearWarnings clears the SQLWarnings associated with
     *            the statement object.  After a call to this method, a call
     *            to getWarnings will return a null SQLWarning object.
     *            (See JDK 1.2.2 API Documentation)
     *
     * @test_Strategy:    Get a Statement object and call clearWarnings() method
     *            on the statement object.Further calling the getWarnings()
     *            method should return a null SQLWarning object
     *
     */

/* throws Exception */
  void StatementTest::testClearWarnings() 
  {
    const sql::SQLWarning * sWarning = stmt->getWarnings();

    if (sWarning != NULL)
    {
        logMsg("Calling clearWarnings method ");
        stmt->clearWarnings();
        sWarning = stmt->getWarnings();

        if (sWarning == NULL)
        {
            logMsg("clearWarnings clears the SQLWarnings");
        }
        else
        {
            logErr("clearWarnings does not clear the SQLWarning");
            FAIL("Call to clearWarnings is Failed!");
        }
    }
    else
    {
        logErr("getWarnings() returns a NULL SQLWarning object");
    }
  }

    /*
     * @testName:         testClose
     * @assertion:        The Statement object provides methods for executing SQL
     *            statements and retrieving results.(See section 40.1 of
     *            JDBC 2.0 API Reference & Tutorial second edition).
     *
     *            The driver must provide full support for Statement methods.
     *            The driver must also support all the methods for executing
     *            SQL Statements in a single batch (Batch Updates). (See
     *            section 6.2.2.3 of Java2 Platform Enterprise Edition(J2EE)
     *            Specification v1.2)
     *
     *            The close method closes the statement object.When a Statement
     *            object is closed, its current ResultSet object, if one exists,
     *            is also closed.  (See JDK 1.2.2 API Documentation)
     *
     * @test_Strategy:    Get a Statement object and call close() method and call
     *            executeQuery() method to check and it should throw SQLException
     *
     */

/* throws Exception */
  void StatementTest::testClose() 
  {
    Statement statemt;
    bool sqlExceptFlag = false;

    statemt.reset( conn->createStatement() );
    logMsg("Calling close method");
    statemt->close();
    String sSelCoffee = sqlProps["SelCoffeeAll"];
    logMsg(String( "Query String : " ) + sSelCoffee);

    try
    {
      rs.reset( statemt->executeQuery(sSelCoffee) );
    }
    catch (sql::SQLException & )
    {
      sqlExceptFlag = true;
    }

    if (sqlExceptFlag)
    {
      logMsg("close method closes the Statement object");
    } else {
      logErr("close method does not close the Statement object");
        FAIL("Call to close method is Failed!");
    }
  }

    /*
     * @testName:         testExecute01
     * @assertion:        The Statement object provides methods for executing SQL
     *            statements and retrieving results.(See section 40.1 of
     *            JDBC 2.0 API Reference & Tutorial second edition).
     *
     *            The driver must provide full support for Statement methods.
     *            The driver must also support all the methods for executing
     *            SQL Statements in a single batch (Batch Updates). (See
     *            section 6.2.2.3 of Java2 Platform Enterprise Edition(J2EE)
     *            Specification v1.2)
     *
     *            The execute(String sql) method returns a boolean value; true
     *            if the first result is ResultSet or false if it is an integer.
     *            (See JDK 1.2.2 API Documentation)
     *
     * @test_Strategy:    Call execute(String sql) of updating a row
     *            It should return a boolean value and the value should be
     *            equal to false
     *
     */

/* throws Exception */
  void StatementTest::testExecute01() 
  {
    bool executeFlag = false;

    String sSqlStmt = sqlProps[ "Upd_Coffee_Tab" ];
    logMsg(String( "SQL Statement to be executed  " ) + sSqlStmt);
    logMsg("Calling execute method ");
    executeFlag = stmt->execute(sSqlStmt);

    if (!executeFlag)
    {
        logMsg("execute method executes the SQL Statement ");
    }
    else
    {
        logErr("execute method does not execute the SQL Statement");
        FAIL("Call to execute is Failed!");
    }
  }

    /*
     * @testName:         testExecute02
     * @assertion:        The Statement object provides methods for executing SQL
     *            statements and retrieving results.(See section 40.1 of
     *            JDBC 2.0 API Reference & Tutorial second edition).
     *
     *            The driver must provide full support for Statement methods.
     *            The driver must also support all the methods for executing
     *            SQL Statements in a single batch (Batch Updates). (See
     *            section 6.2.2.3 of Java2 Platform Enterprise Edition(J2EE)
     *            Specification v1.2)
     *
     *            The execute(String sql) method returns a boolean value;
     *            true if the first result is ResultSet or false if it is
     *            an integer. (See JDK 1.2.2 API Documentation)
     *
     * @test_Strategy:    Get a Statement object and call execute(String sql)
     *            of selecting rows from the database
     *            It should return a boolean value and the value should be equal
     *            to true
     *
     */

/* throws Exception */
  void StatementTest::testExecute02() 
  {
        bool executeFlag = false;

    String sSqlStmt = sqlProps[ "Sel_Coffee_Tab" ];

    logMsg(String( "Sql Statement to be executed  " ) + sSqlStmt);
    logMsg("Calling execute method ");
    executeFlag = stmt->execute(sSqlStmt);
    if (executeFlag) {
        logMsg("execute method executes the SQL Statement ");
    } else {
        logErr("execute method does not execute the SQL Statement");
        FAIL("Call to execute is Failed!");
    }
    }

    /*
     * @testName:         testExecuteQuery01
     * @assertion:        The Statement object provides methods for executing SQL
     *            statements and retrieving results.(See section 40.1 of
     *            JDBC 2.0 API Reference & Tutorial second edition).
     *
     *            The driver must provide full support for Statement methods.
     *            The driver must also support all the methods for executing
     *            SQL Statements in a single batch (Batch Updates). (See
     *            section 6.2.2.3 of Java2 Platform Enterprise Edition(J2EE)
     *            Specification v1.2)
     *
     *            The executeQuery(String sql) method returns a ResultSet object;
     *            It may return an empty ResultSet object but never returns null.
     *            This method throws SQLException if an error occurs in processing
     *            SQL statement or if the SQL statement generates a row count
     *            instead of ResultSet. (See JDK1.2.2 API documentation)
     *
     * @test_Strategy:    Get a Statement object and call executeQuery(String sql)
     *            to select a row from the database
     *            It should return a ResultSet object
     *
     */

/*
 *throws Exception
 * Pretty much useless in C++ */
  void StatementTest::testExecuteQuery01() 
  {
    ResultSet reSet;

    String sSqlStmt = sqlProps[ "SelCoffeeAll" ];
    logMsg(String( "SQL Statement to be executed  :  " ) + sSqlStmt);
    logMsg("Calling executeQuery method ");

    reSet.reset( stmt->executeQuery(sSqlStmt) );

    if ( reSet.get() != NULL )
    {
        logMsg("executeQuery method returns a ResultSet object");
    }
    else
    {
        logErr("executeQuery method does not return a ResultSet object");
        FAIL("Call to executeQuery is Failed!");
    }
  }

    /*
     * @testName:         testExecuteQuery02
     * @assertion:        The Statement object provides methods for executing SQL
     *            statements and retrieving results.(See section 40.1 of
     *            JDBC 2.0 API Reference & Tutorial second edition).
     *
     *            The driver must provide full support for Statement methods.
     *            The driver must also support all the methods for executing
     *            SQL Statements in a single batch (Batch Updates). (See
     *            section 6.2.2.3 of Java2 Platform Enterprise Edition(J2EE)
     *            Specification v1.2)
     *
     *            The executeQuery(String sql) method returns a ResultSet object;
     *            It may return an empty ResultSet object but never returns null.
     *            This method throws SQLException if an error occurs in processing
     *            SQL statement or if the SQL statement generates a row count
     *            instead of ResultSet. (See JDK1.2.2 API documentation)
     *
     * @test_Strategy:    Get a Statement object and call executeQuery(String sql)
     *            to select a non-existent row from the database
     *            It should return a ResultSet object which is empty and call
     *            ResultSet.next() method to check and it should return a false
     *
     */

/* throws Exception */
  void StatementTest::testExecuteQuery02() 
  {
    ResultSet reSet;
    String    sSqlStmt= sqlProps[ "SelCoffeeNull" ];

    logMsg(String( "SQL Statement to be executed  :  " ) + sSqlStmt);
    logMsg("Calling executeQuery method ");

    reSet.reset( stmt->executeQuery(sSqlStmt) );

    if (!reSet->next())
    {
      logMsg(
        "executeQuery method returns an Empty ResultSet for Non-Existent row");
    }
    else
    {
      logErr(
        "executeQuery method does not return an Empty ResultSet for non-existent row");
      FAIL("Call to executeQuery is Failed!");
    }
  }

    /*
     * @testName:         testExecuteQuery03
     * @assertion:        The Statement object provides methods for executing SQL
     *            statements and retrieving results.(See section 40.1 of
     *            JDBC 2.0 API Reference & Tutorial second edition).
     *
     *            The driver must provide full support for Statement methods.
     *            The driver must also support all the methods for executing
     *            SQL Statements in a single batch (Batch Updates). (See
     *            section 6.2.2.3 of Java2 Platform Enterprise Edition(J2EE)
     *            Specification v1.2)
     *
     *            The executeQuery(String sql) method returns a ResultSet object;
     *            It may return an empty ResultSet object but never returns null.
     *            This method throws SQLException if an error occurs in processing
     *            SQL statement or if the SQL statement generates a row count
     *            instead of ResultSet. (See JDK1.2.2 API documentation)
     *
     *
     * @test_Strategy:    Get a Statement object and call executeQuery(String sql)
     *            to insert a row from the database
     *            It should throw SQLException
     *
     */

/* throws Exception */
  void StatementTest::testExecuteQuery03() 
  {
    ResultSet reSet;
    bool      sqlExceptFlag= false;

    String sSqlStmt= sqlProps[ "Ins_Coffee_Tab" ];
    logMsg(String( "SQL Statement to be executed  :  " ) + sSqlStmt);
    
    try
    {
      logMsg("Calling executeQuery method ");
      reSet.reset( stmt->executeQuery(sSqlStmt) );
    }
    catch (sql::SQLException & )
    {
      sqlExceptFlag = true;
    }

    if (!sqlExceptFlag)
    {
      logErr("executeQuery method executes an Insert Statement");
      FAIL("Call to executeQuery is Failed!");
    }
    else
    {
      logMsg(
         "executeQuery method does not execute an Insert Statement");
    }

  }

    /*
     * @testName:         testExecuteUpdate01
     * @assertion:        The Statement object provides methods for executing SQL
     *            statements and retrieving results.(See section 40.1 of
     *            JDBC 2.0 API Reference & Tutorial second edition).
     *
     *            The driver must provide full support for Statement methods.
     *            The driver must also support all the methods for executing
     *            SQL Statements in a single batch (Batch Updates). (See
     *            section 6.2.2.3 of Java2 Platform Enterprise Edition(J2EE)
     *            Specification v1.2)
     *
     *            The executeUpdate(String sql) method returns a integer value;
     *            The value indicates the number of rows affected by INSERT, DELETE
     *            or UPDATE specified in the sql; 0 if no rows were affected or the
     *            statement executed was a DDL statement.
     *            This method throws SQLException if an error occurs in processing
     *            SQL statement or if the SQL statement generates a ResultSet.
     *            (See JDK1.2.2 API documentation)
     *
     * @test_Strategy:    Get a Statement object and call executeUpdate(String sql)
     *            It should return an int value which is equal to row count
     */

/* throws Exception */
  void StatementTest::testExecuteUpdate01() 
  {
    int     updCount=     0;
    int     retRowCount=  0;
    String  sSqlStmt=     sqlProps[ "Upd_Coffee_Tab" ];

    logMsg(String( "Update String  : " ) + sSqlStmt);

    logMsg("Calling executeUpdate method ");
    updCount = stmt->executeUpdate(sSqlStmt);

    String countQuery= sqlProps[ "Coffee_Updcount_Query" ];

    logMsg( String( "Query String :  " ) + countQuery );
    rs.reset( stmt->executeQuery(countQuery) );

    rs->next();
    retRowCount = rs->getInt(1);

    TestsListener::theInstance().messagesLog()
      << "Number of rows in the table with the specified condition  "
      << retRowCount << std::endl;

    if (updCount == retRowCount)
    {
      logMsg("executeUpdate executes the SQL Statement ");
    }
    else
    {
      logErr("executeUpdate does not execute the SQL Statement ");
      FAIL("Call to executeUpdate is Failed!");
    }
  }

    /*
     * @testName:         testExecuteUpdate03
     * @assertion:        The Statement object provides methods for executing SQL
     *            statements and retrieving results.(See section 40.1 of
     *            JDBC 2.0 API Reference & Tutorial second edition).
     *
     *            The driver must provide full support for Statement methods.
     *            The driver must also support all the methods for executing
     *            SQL Statements in a single batch (Batch Updates). (See
     *            section 6.2.2.3 of Java2 Platform Enterprise Edition(J2EE)
     *            Specification v1.2)
     *
     *            The executeUpdate(String sql) method returns a integer value;
     *            The value indicates the number of rows affected by INSERT, DELETE
     *            or UPDATE specified in the sql; 0 if no rows were affected or the
     *            statement executed was a DDL statement.
     *            This method throws SQLException if an error occurs in processing
     *            SQL statement or if the SQL statement generates a ResultSet.
     *            (See JDK1.2.2 API documentation)
     *
     * @test_Strategy:    Get a Statement object and call executeUpdate(String sql)
     *            for selecting row from the table
     *            It should throw a SQL Exception
     *
     */

/* throws Exception */
  void StatementTest::testExecuteUpdate03() 
  {
        bool sqlExceptFlag = false;
        int updCount = 0;

    String sSqlStmt = sqlProps[ "Sel_Coffee_Tab" ];
    logMsg(String( "SQL String of non - existent row  :  " ) + sSqlStmt);

    try
    {
        logMsg("Calling executeUpdate method ");
        updCount = stmt->executeUpdate(sSqlStmt);
    }
    catch ( sql::SQLException & )
    {
        sqlExceptFlag = true;
    }

    if (sqlExceptFlag)
    {
        logMsg(
            "executeUpdate does not execute the SQL statement on non-existent row");
    }
    else
    {
        logErr(
            "executeUpdate executes the SQL statement on non-existent row");
        FAIL("Call to executeUpdate is Failed!");
    }
  }

    /*
     * @testName:         testGetFetchDirection
     * @assertion:        The Statement object provides methods for executing SQL
     *            statements and retrieving results.(See section 40.1 of
     *            JDBC 2.0 API Reference & Tutorial second edition).
     *
     *            The driver must provide full support for Statement methods.
     *            The driver must also support all the methods for executing
     *            SQL Statements in a single batch (Batch Updates). (See
     *            section 6.2.2.3 of Java2 Platform Enterprise Edition(J2EE)
     *            Specification v1.2)
     *
     *            The getFetchDirection() method returns a integer value;
     *            The value that is been set by the setFetchDirection method.
     *            If no fetch direction has been set, the return value is
     *            implementation specific. (See JDK1.2.2 API documentation)
     *
     * @test_Strategy:    Get a Statement object and call the getFetchDirection() method
     *            It should return a int value and the value should be equal to
     *            any of the values FETCH_FORWARD or FETCH_REVERSE or FETCH_UNKNOWN
     *
     */

/* throws Exception 
 *
 * Not included into sql::statement (and thus into MySQL_Statement)
 * Disabled so far.
 */

#ifdef MISSING_METHODS_INCLUDED2STATEMENT
  void StatementTest::testGetFetchDirection() 
  {
    int fetchDirVal = 0;

    logMsg("Calling getFetchDirection method ");

    fetchDirVal = stmt->getFetchDirection();

    if (fetchDirVal == ResultSet::FETCH_FORWARD)
    {
      logMsg(
            "getFetchDirection method returns ResultSet.FETCH_FORWARD ");
    } else if (fetchDirVal == ResultSet.FETCH_REVERSE) {
        logMsg(
            "getFetchDirection method returns ResultSet.FETCH_REVERSE");
    } else if (fetchDirVal == ResultSet.FETCH_UNKNOWN) {
        logMsg(
            "getFetchDirection method returns ResultSet.FETCH_UNKNOWN");
    } else {
        logErr(" getFetchDirection method returns a invalid value");
        FAIL("Call to getFetchDirection is Failed");
    }
  }
#endif


    /*
     * @testName:         testGetMoreResults01
     * @assertion:        The Statement object provides methods for executing SQL
     *            statements and retrieving results.(See section 40.1 of
     *            JDBC 2.0 API Reference & Tutorial second edition).
     *
     *            The driver must provide full support for Statement methods.
     *            The driver must also support all the methods for executing
     *            SQL Statements in a single batch (Batch Updates). (See
     *            section 6.2.2.3 of Java2 Platform Enterprise Edition(J2EE)
     *            Specification v1.2)
     *
     *            The getMoreResults() method returns a boolean value;
     *            true if the next result is ResultSet object; false if it is
     *            an integer indicating that it is an update count or there are
     *            no more results. There are no more results when the following
     *            condition is satisfied.
     *            (getMoreResults==false && getUpdatecount==-1)
     *            (See JDK1.2.2 API documentation)
     *
     * @test_Strategy:    Get a Statement object and call the execute() method for
     *            selecting a row and call getMoreResults() method
     *            It should return a boolean value
     *
     */

/* throws Exception */
  void StatementTest::testGetMoreResults01() 
  {

    String sSqlStmt = sqlProps[ "SelCoffeeAll" ];

    logMsg(String( "Query String : " ) + sSqlStmt);
    stmt->execute(sSqlStmt);

    logMsg("Calling getMoreResults on Statement");
    bool moreResVal = stmt->getMoreResults();

    if ((moreResVal == true) || (moreResVal == false))
    {
      TestsListener::theInstance().messagesLog()
        << "getMoreResults method returns :" << moreResVal << std::endl;
    }
    else
    {
        logErr(" getMoreResults method returns a invalid value");
        FAIL("Call to getMoreResults is Failed!");
    }
  }

    /*
     * @testName:         testGetMoreResults02
     * @assertion:        The Statement object provides methods for executing SQL
     *            statements and retrieving results.(See section 40.1 of
     *            JDBC 2.0 API Reference & Tutorial second edition).
     *
     *            The driver must provide full support for Statement methods.
     *            The driver must also support all the methods for executing
     *            SQL Statements in a single batch (Batch Updates). (See
     *            section 6.2.2.3 of Java2 Platform Enterprise Edition(J2EE)
     *            Specification v1.2)
     *
     *            The getMoreResults() method returns a boolean value;
     *            true if the next result is ResultSet object; false if it is
     *            an integer indicating that it is an update count or there are
     *            no more results. There are no more results when the following
     *            condition is satisfied.
     *            (getMoreResults==false && getUpdatecount==-1)
     *            (See JDK1.2.2 API documentation)
     *
     * @test_Strategy:    Get a Statement object and call the execute() method for
     *            selecting a non-existent row and call getMoreResults() method
     *            It should return a boolean value and the value should be
     *            equal to false
     *
     */

/* throws Exception */
  void StatementTest::testGetMoreResults02() 
  {

    String sSqlStmt = sqlProps[ "SelCoffeeNull" ];

    logMsg(String( "Query String : " ) + sSqlStmt);
    stmt->execute(sSqlStmt);

    logMsg("Calling getMoreResults on Statement");
    bool moreResVal = stmt->getMoreResults();

    if (!moreResVal)
    {
      TestsListener::theInstance().messagesLog()
        << "getMoreResults method returns :" << moreResVal << std::endl;
    }
    else
    {
        logErr(" getMoreResults method returns a invalid value");
        FAIL("Call to getMoreResults is Failed!");
    }
  }

    /*
     * @testName:         testGetMoreResults03
     * @assertion:        The Statement object provides methods for executing SQL
     *            statements and retrieving results.(See section 40.1 of
     *            JDBC 2.0 API Reference & Tutorial second edition).
     *
     *            The driver must provide full support for Statement methods.
     *            The driver must also support all the methods for executing
     *            SQL Statements in a single batch (Batch Updates). (See
     *            section 6.2.2.3 of Java2 Platform Enterprise Edition(J2EE)
     *            Specification v1.2)
     *
     *            The getMoreResults() method returns a boolean value;
     *            true if the next result is ResultSet object; false if it is
     *            an integer indicating that it is an update count or there are
     *            no more results. There are no more results when the following
     *            condition is satisfied.
     *            (getMoreResults==false && getUpdatecount==-1)
     *            (See JDK1.2.2 API documentation)
     *
     * @test_Strategy:    Get a Statement object and call the execute() method for
     *            updating a row and call getMoreResults() method
     *            It should return a boolean value and the value should be
     *            equal to false
     *
     */

/* throws Exception */
  void StatementTest::testGetMoreResults03() 
  {

    String sSqlStmt = sqlProps[ "Upd_Coffee_Tab" ];

    logMsg(String( "Query String : " ) + sSqlStmt);
    stmt->execute(sSqlStmt);

    logMsg("Calling getMoreResults on Statement");
    bool moreResVal = stmt->getMoreResults();

    if (!moreResVal)
    {
      TestsListener::theInstance().messagesLog()
        << "getMoreResults method returns :" << moreResVal << std::endl;
    }
    else
    {
        logErr(" getMoreResults method returns a invalid value");
        FAIL("Call to getMoreResults is Failed!");
    }
  }


    /*
     * @testName:         testGetResultSet01
     * @assertion:        The Statement object provides methods for executing SQL
     *            statements and retrieving results.(See section 40.1 of
     *            JDBC 2.0 API Reference & Tutorial second edition).
     *
     *            The driver must provide full support for Statement methods.
     *            The driver must also support all the methods for executing
     *            SQL Statements in a single batch (Batch Updates). (See
     *            section 6.2.2.3 of Java2 Platform Enterprise Edition(J2EE)
     *            Specification v1.2)
     *
     *            The getResultSet() method returns a ResultSet object;
     *            the current result set as a ResultSet object; null if the
     *            result is an integer indicating that it is an update count
     *            or there are no more results.(See JDK1.2.2 API documentation)
     *
     * @test_Strategy:    Get a Statement object and call execute() method for
     *            selecting a row and call getResultSet() method
     *            It should return a ResultSet object
     *
     */

/* throws Exception */
  void StatementTest::testGetResultSet01() 
  {
    ResultSet retResSet /*= NULL*/;
    String    sSqlStmt  = sqlProps[ "SelCoffeeAll" ];

    logMsg(String( "Query String :  " ) + sSqlStmt);
    logMsg("Calling getResultSet on Statement");
    stmt->execute(sSqlStmt);

    retResSet.reset( stmt->getResultSet() );

    if ( retResSet.get() != NULL)
    {
        logMsg("getResultSet method returns a ResultSet object ");
    }
    else
    {
        logErr(
            " getResultSet method does not return a ResultSet object");
        FAIL("Call to getResultSet is Failed!");
    }
  }

    /*
     * @testName:         testGetResultSet02
     * @assertion:        The Statement object provides methods for executing SQL
     *            statements and retrieving results.(See section 40.1 of
     *            JDBC 2.0 API Reference & Tutorial second edition).
     *
     *            The driver must provide full support for Statement methods.
     *            The driver must also support all the methods for executing
     *            SQL Statements in a single batch (Batch Updates). (See
     *            section 6.2.2.3 of Java2 Platform Enterprise Edition(J2EE)
     *            Specification v1.2)
     *
     *            The getResultSet() method returns a ResultSet object;
     *            the current result set as a ResultSet object; null if the
     *            result is an integer indicating that it is an update count
     *            or there are no more results.(See JDK1.2.2 API documentation)
     *
     * @test_Strategy:    Get a Statement object and call execute() method for
     *            updating a row.Then call getResultSet() method
     *            It should return a  Null ResultSet object
     */

/* throws Exception */
  void StatementTest::testGetResultSet02() 
  {
    ResultSet retResSet /*= NULL*/;
    String    sSqlStmt  = sqlProps[ "Upd_Coffee_Tab" ];

    logMsg(String( "Query String :  " ) + sSqlStmt);
    logMsg("Calling getResultSet on Statement");

    stmt->execute(sSqlStmt);
    retResSet.reset( stmt->getResultSet() );

    if (retResSet.get() == NULL)
    {
      logMsg("getResultSet method returns a Null ResultSet object ");
    }
    else
    {
      logErr(
        " getResultSet method does (not) return a ResultSet object");
      FAIL("Call to getResultSet is Failed!");
    }
  }

    /*
     * @testName:         testGetResultSetConcurrency01
     * @assertion:        The Statement object provides methods for executing SQL
     *            statements and retrieving results.(See section 40.1 of
     *            JDBC 2.0 API Reference & Tutorial second edition).
     *
     *            The driver must provide full support for Statement methods.
     *            The driver must also support all the methods for executing
     *            SQL Statements in a single batch (Batch Updates). (See
     *            section 6.2.2.3 of Java2 Platform Enterprise Edition(J2EE)
     *            Specification v1.2)
     *
     *            The getResultSetConcurrency() method returns an integer value;
     *            the value representing the concurrency mode for the ResultSet
     *            objects and the value can be any one of the following
     *            ResultSet.CONCUR_READ_ONLY and ResultSet.CONCUR_UPDATABLE.
     *            (See JDK1.2.2 API documentation)
     *
     * @test_Strategy:    Get a Statement object and call getResultSetConcurrency() method
     *            It should return an int value either CONCUR_READ_ONLY or
     *            CONCUR_UPDATABLE.
     */

/* getResultSetConcurrency not included into statement interface
 *
 *
 * throws Exception
 */

#ifdef MISSING_METHODS_INCLUDED2STATEMENT
  void StatementTest::testGetResultSetConcurrency01() 
  {
        int rsConcur = 0;

    rsConcur = stmt->getResultSetConcurrency();
    if ((rsConcur == sql::ResultSet::CONCUR_READ_ONLY)
            || (rsConcur == sql::ResultSet::CONCUR_UPDATABLE)) {
        logMsg(
            "getResultSetConcurrency method returns ResultSet Concurrency mode  "
            + rsConcur);
    } else {
        logErr(
            " getResultSetConcurrency method does not return a valid value");
        FAIL("Call to getResultSetConcurrency is Failed!");
    }
  }


    /*
     * @testName:         testGetResultSetType01
     * @assertion:        The Statement object provides methods for executing SQL
     *            statements and retrieving results.(See section 40.1 of
     *            JDBC 2.0 API Reference & Tutorial second edition).
     *
     *            The driver must provide full support for Statement methods.
     *            The driver must also support all the methods for executing
     *            SQL Statements in a single batch (Batch Updates). (See
     *            section 6.2.2.3 of Java2 Platform Enterprise Edition(J2EE)
     *            Specification v1.2)
     *
     *            The getResultSetType() method returns an integer value;
     *            the value representing the type of the ResultSet objects
     *            and the value can be any one of the following
     *            ResultSet.TYPE_FORWARD_ONLY, ResultSet.TYPE_SCROLL_SENSITIVE
     *            and ResultSet.TYPE_SCROLL_INSENSITIVE.
     *            (See JDK1.2.2 API documentation)
     *
     * @test_Strategy:    Get a Statement object and call getResultSetType() method
     *            It should return an int value which should be either
     *            TYPE_FORWARD_ONLY or TYPE_SCROLL_INSENSITIVE or TYPE_SCROLL_SENSITIVE
     */

/* throws Exception */
  void StatementTest::testGetResultSetType01() 
  {
        int rsType = 0;

    rsType = stmt->getResultSetType();
    if (rsType == sql::ResultSet::TYPE_FORWARD_ONLY) {
        logMsg("getResultSetType method returns TYPE_FORWARD_ONLY"
            + rsType);
    } else if (rsType == sql::ResultSet::TYPE_SCROLL_INSENSITIVE) {
        logMsg(
            "getResultSetType method returns TYPE_SCROLL_INSENSITIVE "
            + rsType);
    } else if (rsType == sql::ResultSet::TYPE_SCROLL_SENSITIVE) {
        logMsg(
            "getResultSetType method returns TYPE_SCROLL_SENSITIVE  "
            + rsType);
    } else {
        logErr(" getResultSetType method does not return a valid value");
        FAIL("Call to getResultSetType is Failed!");
    }
  }

    /*
     * @testName:         testGetResultSetType02
     * @assertion:        The Statement object provides methods for executing SQL
     *            statements and retrieving results.(See section 40.1 of
     *            JDBC 2.0 API Reference & Tutorial second edition).
     *
     *            The driver must provide full support for Statement methods.
     *            The driver must also support all the methods for executing
     *            SQL Statements in a single batch (Batch Updates). (See
     *            section 6.2.2.3 of Java2 Platform Enterprise Edition(J2EE)
     *            Specification v1.2)
     *
     *            The getResultSetType() method returns an integer value;
     *            the value representing the type of the ResultSet objects
     *            and the value can be any one of the following
     *            ResultSet.TYPE_FORWARD_ONLY, ResultSet.TYPE_SCROLL_SENSITIVE
     *            and ResultSet.TYPE_SCROLL_INSENSITIVE.
     *            (See JDK1.2.2 API documentation)
     *
     * @test_Strategy:    Call Connection.createStatement with the Type mode as
     *            TYPE_FORWARD_ONLY and call getResultSetType() method
     *            It should return a int value and the value should be equal
     *            to ResultSet.TYPE_FORWARD_ONLY
     */

/* throws Exception */
  void StatementTest::testGetResultSetType02() 
  {
        int rsType = 0;
        Statement statemt /*= NULL*/;

    logMsg("Creating Statement object with the ResultSet Type and Type");
    statemt = conn->createStatement(sql::ResultSet::TYPE_FORWARD_ONLY,
            sql::ResultSet::CONCUR_READ_ONLY);
    rsType = statemt->getResultSetType();
    if (rsType == sql::ResultSet::TYPE_FORWARD_ONLY) {
        logMsg("getResultSetType method returns TYPE_FORWARD_ONLY "
            + rsType);
    } else {
        statemt->close();
        logErr(" getResultSetType method does not return a valid value");
        FAIL("Call to getResultSetType is Failed!");
    }
    statemt->close();
    }

    /*
     * @testName:         testGetResultSetType03
     * @assertion:        The Statement object provides methods for executing SQL
     *            statements and retrieving results.(See section 40.1 of
     *            JDBC 2.0 API Reference & Tutorial second edition).
     *
     *            The driver must provide full support for Statement methods.
     *            The driver must also support all the methods for executing
     *            SQL Statements in a single batch (Batch Updates). (See
     *            section 6.2.2.3 of Java2 Platform Enterprise Edition(J2EE)
     *            Specification v1.2)
     *
     *            The getResultSetType() method returns an integer value;
     *            the value representing the type of the ResultSet objects
     *            and the value can be any one of the following
     *            ResultSet.TYPE_FORWARD_ONLY, ResultSet.TYPE_SCROLL_SENSITIVE
     *            and ResultSet.TYPE_SCROLL_INSENSITIVE.
     *            (See JDK1.2.2 API documentation)
     *
     * @test_Strategy:    Call Connection.createStatement with the Type mode as
     *            TYPE_SCROLL_INSENSITIVE and call getResultSetType() method
     *            It should return a int value and the value should be equal
     *            to ResultSet.TYPE_SCROLL_INSENSITIVE
     */

/* throws Exception */
  void StatementTest::testGetResultSetType03() 
  {
        int rsType = 0;
        Statement statemt /*= NULL*/;

    logMsg("Creating Statement object with the ResultSet Type and Type");
    statemt = conn->createStatement(sql::ResultSet::TYPE_SCROLL_INSENSITIVE,
            sql::ResultSet::CONCUR_READ_ONLY);
    rsType = statemt->getResultSetType();
    if (rsType == sql::ResultSet::TYPE_SCROLL_INSENSITIVE) {
        logMsg(
            "getResultSetType method returns TYPE_SCROLL_INSENSITIVE "
            + rsType);
    } else {
        statemt->close();
        logErr(" getResultSetType method does not return a valid value");
        FAIL("Call to getResultSetType is Failed!");
    }
    statemt->close();
  }

    /*
     * @testName:         testGetUpdateCount01
     *
     * @assertion:        A driver must provide support for Statement and
     *            ResultSet.  This implies that the methods in the
     *            Statement interface must be implemented and must behave as
     *            specified in the JDBC 1.0 and 2.0 specifications.        (See
     *            Section :40.3 Statement Methods JDBC 2.0 API Tutorial
     *            & Reference).
     *
     *            The driver must provide full support for Statement methods.
     *            The driver must also support all the methods for executing
     *            SQL Statements in a single batch (Batch Updates). (See
     *            section 6.2.2.3 of Java2 Platform Enterprise Edition(J2EE)
     *            Specification v1.2)
     *
     *            The getUpdateCount() method should return a integer value;
     *            the value might be greater than 0 representing the rows affected;
     *            0 if no rows are affected or if DDL statement; -1 if the result
     *            is a ResultSet object or there are no more results
     *            (See JDK 1.2.2 API Documentation)
     *
     * @test_Strategy:    Get a Statement object and call the execute() method for
     *            updating a row and call getUpdateCount() method
     *            It should return a int value and the value should be
     *            equal to number of rows with the specified condition for update
     */
#endif


/* throws Exception */
  void StatementTest::testGetUpdateCount01() 
  {
    int updCountVal   = 0;
    int rowsAffectVal = 0;
    String sSqlStmt   = sqlProps[ "Upd_Coffee_Tab" ];

    logMsg(String( "Query String : " ) + sSqlStmt);
    stmt->execute(sSqlStmt);

    logMsg("Calling getUpdateCount on Statement");
    updCountVal = static_cast<int>( stmt->getUpdateCount() );

    String sQuery = sqlProps[ "Coffee_Updcount_Query" ];
    logMsg(String( "Query String : " ) + sQuery);

    ResultSet rs1( stmt->executeQuery(sQuery) );
    rs1->next();

    rowsAffectVal = rs1->getInt(1);

    TestsListener::theInstance().messagesLog()
      <<"Number of Rows Affected by Update Statement " << rowsAffectVal
      << std::endl;

    if (updCountVal == rowsAffectVal)
    {
      TestsListener::theInstance().messagesLog()
        << "getUpdateCount method returns :" << updCountVal << std :: endl;
    }
    else
    {
        logErr(" getUpdateCount method returns a invalid value");
        FAIL("Call to getUpdateCount is Failed!");
    }
  }

    /*
     * @testName:         testGetUpdateCount02
     *
     * @assertion:        A driver must provide support for Statement and
     *            ResultSet.  This implies that the methods in the
     *            Statement interface must be implemented and must behave as
     *            specified in the JDBC 1.0 and 2.0 specifications.        (See
     *            Section :40.3 Statement Methods JDBC 2.0 API Tutorial
     *            & Reference).
     *
     *            The driver must provide full support for Statement methods.
     *            The driver must also support all the methods for executing
     *            SQL Statements in a single batch (Batch Updates). (See
     *            section 6.2.2.3 of Java2 Platform Enterprise Edition(J2EE)
     *            Specification v1.2)
     *
     *            The getUpdateCount() method should return a integer value;
     *            the value might be greater than 0 representing the rows affected;
     *            0 if no rows are affected or if DDL statement; -1 if the result
     *            is a ResultSet object or there are no more results
     *            (See JDK 1.2.2 API Documentation)
     *
     * @test_Strategy:    Get a Statement object and call the execute() method for
     *            selecting a non-existent row and call getUpdateCount() method
     *            It should return a int value and the value should be
     *            equal to -1
     */

/* throws Exception */
  void StatementTest::testGetUpdateCount02() 
  {
    int updCountVal = 0;
    String sSqlStmt = sqlProps[ "SelCoffeeNull" ];

    logMsg(String( "Query String : " ) + sSqlStmt);
    stmt->execute(sSqlStmt);

    logMsg("Calling getMoreResults on Statement");
    updCountVal = static_cast<int>( stmt->getUpdateCount() );

    if (updCountVal == -1)
    {
        logMsg( "getUpdateCount method returns : -1" );
    }
    else
    {
        logErr(" getUpdateCount method returns a invalid value");
        FAIL("Call to getUpdateCount is Failed!");
    }
    }

    /*
     * @testName:         testGetWarnings
     *
     * @assertion:        A driver must provide support for Statement and
     *            ResultSet.  This implies that the methods in the
     *            Statement interface must be implemented and must behave as
     *            specified in the JDBC 1.0 and 2.0 specifications.        (See
     *            Section :40.3 Statement Methods JDBC 2.0 API Tutorial
     *            & Reference).
     *
     *            The driver must provide full support for Statement methods.
     *            The driver must also support all the methods for executing
     *            SQL Statements in a single batch (Batch Updates). (See
     *            section 6.2.2.3 of Java2 Platform Enterprise Edition(J2EE)
     *            Specification v1.2)
     *
     *            The getWarnings() method should return a SQLWarning object;
     *            or null if there are no warnings (See JDK 1.2.2 API
     *            Documentation)
     *
     * @test_Strategy:    Get a Statement object and call getWarnings() method
     *            should return an SQLWarning object
     */

/* throws Exception */
  void StatementTest::testGetWarnings() 
  {
    if (hasSps)
    {
      initTable("Integer_Tab", sqlProps, conn);

      //TODO: Need to invent somth to generate warnings instead of this
#ifdef  WE_GOT_SMTH2DEAL_WITH_SP
    
      CallableStatement cstmt = conn->prepareCall(
      "{call Integer_Proc(?,?,?)}");
      logMsg(String( "The Callable Statement " ) + cstmt);
      cstmt->registerOutParameter(1, "cppconn/datatype.h".INTEGER);
      cstmt->registerOutParameter(2, "cppconn/datatype.h".INTEGER);
      cstmt->registerOutParameter(3, "cppconn/datatype.h".INTEGER);
      cstmt->execute();

      int nRetVal = cstmt->getInt(1);

      Statement state( cstmt );
#else
      sql::Statement * state = stmt.get();
#endif

      logMsg("Calling getWarnings method");
      const sql::SQLWarning * sWarning = state->getWarnings();

      if (sWarning != NULL)
      {
        logMsg("getWarnings method returns a SQLWarning object");
      }
      else if (sWarning == NULL)
      {
        logErr( "getWarnings() method returns a NULL SQLWarning Object");
      }

      clearTable("Integer_Tab", conn);
    }
  }

    /*
     * @testName:         testSetFetchDirection04
     * @assertion:        A driver must provide support for Statement and
     *            ResultSet.  This implies that the methods in the
     *            Statement interface must be implemented and must behave as
     *            specified in the JDBC 1.0 and 2.0 specifications.        (See
     *            Section :40.3 Statement Methods JDBC 2.0 API Tutorial
     *            & Reference).
     *
     *            The driver must provide full support for Statement methods.
     *            The driver must also support all the methods for executing
     *            SQL Statements in a single batch (Batch Updates). (See
     *            section 6.2.2.3 of Java2 Platform Enterprise Edition(J2EE)
     *            Specification v1.2)
     *
     *            The setFetchDirection(int dir) method sets the statement
     *            object's fetch direction.  The setFetchDirection method does
     *            not return any value. (See JDK 1.2.2 API Documentation)
     *
     * @test_Strategy:    Get a Statement object and call the setFetchDirection(int direction)
     *            method with an invalid value and it should throw an SQLException
     */

/* throws Exception */
#ifdef MISSING_METHODS_INCLUDED2STATEMENT
  void StatementTest::testSetFetchDirection04() 
  {
    bool sqlExceptFlag = false;

    logMsg("Calling setFetchDirection method ");
    
    try
    {
        stmt->setFetchDirection(-1);
    }
    catch (sql::SQLException * sqe)
    {
        sqlExceptFlag = true;
    }

    if (sqlExceptFlag) {
        logMsg(
            "setFetchDirection method does not sets the invalid direction for the ResultSet ");
    } else {
        logErr(
            "setFetchDirection method sets the invalid direction for ResultSet");
        FAIL("Call to setFetchDirection is Failed");
    }
  }
#endif

}
}
